В проекте **Spring PetClinic** пакет `owner` содержит классы, связанные с управлением владельцами питомцев и их питомцами. Рассмотрим подробно каждый файл из этого пакета, описывая его назначение и основные элементы кода.

### 1. Owner.java

Класс `Owner` представляет владельца питомца и наследует свойства от класса `Person`. Он содержит информацию о владельце и список его питомцев.

**Код класса:**


```java
@Entity
@Table(name = "owners")
public class Owner extends Person {

    @Column(name = "address")
    @NotEmpty
    private String address;

    @Column(name = "city")
    @NotEmpty
    private String city;

    @Column(name = "telephone")
    @NotEmpty
    @Digits(fraction = 0, integer = 10)
    private String telephone;

    @OneToMany(cascade = CascadeType.ALL, mappedBy = "owner")
    private Set<Pet> pets = new HashSet<>();

    // Геттеры и сеттеры
}
```


**Назначение кода:**

- Аннотации `@Entity` и `@Table(name = "owners")` указывают, что класс является сущностью JPA и сопоставляется с таблицей `owners` в базе данных.
- Поля `address`, `city` и `telephone` хранят контактную информацию владельца. Аннотации `@NotEmpty` и `@Digits` обеспечивают валидацию данных.
- Поле `pets` представляет собой коллекцию питомцев, принадлежащих владельцу, с каскадным типом `ALL`, что означает, что все операции (сохранение, обновление, удаление) над владельцем будут применяться и к его питомцам.

**Зачем это нужно:**

Этот класс позволяет хранить и управлять информацией о владельцах и их питомцах, обеспечивая связь между ними.

### 2. Pet.java

Класс `Pet` представляет питомца и содержит информацию о нем, включая тип, дату рождения и историю посещений.

**Код класса:**


```java
@Entity
@Table(name = "pets")
public class Pet extends NamedEntity {

    @Column(name = "birth_date")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private LocalDate birthDate;

    @ManyToOne
    @JoinColumn(name = "type_id")
    private PetType type;

    @ManyToOne
    @JoinColumn(name = "owner_id")
    private Owner owner;

    @OneToMany(cascade = CascadeType.ALL, mappedBy = "pet")
    private Set<Visit> visits = new LinkedHashSet<>();

    // Геттеры и сеттеры
}
```


**Назначение кода:**

- Аннотации `@Entity` и `@Table(name = "pets")` указывают, что класс является сущностью JPA и сопоставляется с таблицей `pets`.
- Поле `birthDate` хранит дату рождения питомца с форматом `yyyy-MM-dd`.
- Поле `type` с аннотацией `@ManyToOne` устанавливает связь с типом питомца (`PetType`).
- Поле `owner` с аннотацией `@ManyToOne` устанавливает связь с владельцем (`Owner`).
- Поле `visits` представляет коллекцию посещений питомца, с каскадным типом `ALL`.

**Зачем это нужно:**

Этот класс позволяет хранить детальную информацию о каждом питомце, включая его тип, владельца и историю посещений, что важно для ведения медицинских записей.

### 3. PetType.java

Класс `PetType` определяет тип питомца, например, собака или кошка.

**Код класса:**


```java
@Entity
@Table(name = "types")
public class PetType extends NamedEntity {
    // Дополнительные поля и методы могут быть добавлены при необходимости
}
```


**Назначение кода:**

- Аннотации `@Entity` и `@Table(name = "types")` указывают на соответствие таблице `types`.
- Наследование от `NamedEntity` предполагает наличие поля `name` для хранения названия типа питомца.

**Зачем это нужно:**

Класс используется для классификации питомцев по видам, что упрощает управление и фильтрацию данных.

### 4. Visit.java

Класс `Visit` представляет посещение питомцем ветеринарной клиники.

**Код класса:**


```java
@Entity
@Table(name = "visits")
public class Visit extends BaseEntity {

    @Column(name = "visit_date")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private LocalDate date;

    @Column(name = "description")
    @NotEmpty
    private String description;

    @ManyToOne
    @JoinColumn(name = "pet_id")
    private Pet pet;

    // Геттеры и сеттеры
}
```


**Назначение кода:**

- Аннотации `@Entity` и `@Table(name = "visits")` указывают на соответствие таблице `visits`.
- Поле `date` хранит дату визита.
- Поле `description` содержит описание причины визита.
- Поле `pet` с аннотацией `@ManyToOne` устанавливает связь с питомцем (`Pet`).

**Зачем это нужно:**

Класс позволяет отслеживать историю медицинских посещений каждого питомца, что важно для ведения медицинских записей и планирования последующего ухода.

### 5. OwnerController.java

Класс `OwnerController` отвечает за обработку HTTP-запросов, связанных с владельцами питомцев.

**Код класса:**


```java
@Controller
@RequestMapping("/owners")
class OwnerController {

    private static final String VIEWS_OWNER_CREATE_OR_UPDATE_FORM = "owners/createOr 